
package java.security;

public class InvalidAlgorithmParameterException extends GeneralSecurityException {
    public InvalidAlgorithmParameterException()
    {
    }

    public InvalidAlgorithmParameterException(String msg)
    {
        super(msg);
    }
}
